import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Dictionary;

import java.io.*;

/**
 * A simple sound player. To start, create an instance of this class.
 * 
 * The sound player can play sound clips in WAV, AU and AIFF formats
 * with standard sample rates.
 * 
 * @author Michael Kolling, David J Barnes and Poul Henriksen
 * @version 1.0
 */
public class SoundPlayerGUI extends JFrame
    implements ChangeListener, ActionListener
{
    private static final String VERSION = "Version 1.0";
    private static final String DEFAULT_AUDIO_DIR = "audio";
    
    private JList fileList;
    private JSlider slider;
    private JSlider volumeSlider;
    private JLabel infoLabel;
    private SoundEngine player;
    private String format = null;
    private File currentDir;
    private JFileChooser chooser;
    
    private JButton playButton = new JButton("Play");    
    private JButton stopButton = new JButton("Stop");
    private JButton pauseButton = new JButton("Pause");
    private JButton resumeButton = new JButton("Resume");

    /**
     * Main method for starting the player from a command line.
     */
    public static void main(String[] args)
    {
        SoundPlayerGUI gui = new SoundPlayerGUI();
    }
    
    /**
     * Create a SoundPlayer and display its GUI on screen.
     */
    public SoundPlayerGUI()
    {
        super("SoundPlayer");
        player = new SoundEngine();        
        makeFrame();
    }

    /**
     * Play the sound file currently selected in the file list. If there is no
     * selection in the list, or if the selected file is not a sound file, 
     * do nothing.
     */
    private void play()
    {
        String filename = (String)fileList.getSelectedValue();
        if(filename == null) {  // nothing selected
            return;
        }
        
        
        boolean successful = player.loadSound(new File(currentDir, filename));
      
        if(successful) {
            int duration = player.getDuration();
            showInfo(filename + " (" + duration+ " seconds)");
            slider.setMaximum(duration);  
            
            int increment = duration/4; //create 4 marks
            if(increment < 1) {
                increment = 1;
            }
            Dictionary table = slider.createStandardLabels(increment);
            table.put(new Integer(duration), new JLabel("" + duration));
            slider.setLabelTable(table);
            slider.setMajorTickSpacing(increment);
            slider.setValue(0);
            
            pauseButton.setEnabled(true);
            stopButton.setEnabled(true);
            player.play();
        }
        else {
            showInfo("Could not play file - unknown format");
        }
    }

    /**
     * Display information about a selected sound file (name and clip length).
     */
    private void showInfo(String message)
    {
        infoLabel.setText(message);
    }
    
    /**
     * Stop the currently playing sound file (if there is one playing).
     */
    private void stop()
    {
        stopButton.setEnabled(false);
        pauseButton.setEnabled(false);
        player.stop();
    }

    /**
     * Stop the currently playing sound file (if there is one playing).
     */
    private void pause()
    {
        resumeButton.setEnabled(true);
        pauseButton.setEnabled(false);
        player.pause();
    }

    /**
     * Resume a previously suspended sound file.
     */
    private void resume()
    {        
        resumeButton.setEnabled(false);
        pauseButton.setEnabled(true);
        player.resume();
    }

    /**
     * Quit function: quit the application.
     */
    private void quit()
    {
        System.exit(0);
    }
    
    
    /**
     * 'About' function: show the 'about' box.
     */
    private void showAbout()
    {
        JOptionPane.showMessageDialog(this, 
                    "SoundPlayer\n" + VERSION,
                    "About SoundPlayer", 
                    JOptionPane.INFORMATION_MESSAGE);
    }
    
    /**
     * Opens a filebrowser to select either a single file or a directory of files.
     */
    private void open() {
        if(chooser == null) {
            chooser = new JFileChooser(DEFAULT_AUDIO_DIR);
            chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        }
        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION) {
            currentDir = chooser.getSelectedFile();
            reload();
        }
    }
    
    
    /**
     * Reloads the files
     */
    private void reload() {
        disableButtons();
        if(currentDir == null) {
            currentDir = new File(DEFAULT_AUDIO_DIR);
        }
        if(currentDir.isDirectory()) {
            fileList.setListData(findFiles(currentDir, format)); 
        }
        else {
            String filename = currentDir.getName();
            currentDir = currentDir.getParentFile();
            fileList.setListData(new String[]{filename});
        }
    }
    
    /**
     * Load the file names of all files in the given directory.
     */
    private String[] findFiles(File dir, String suffix)
    {
        if(dir.isDirectory()) {
            String[] allFiles = dir.list();
            if(suffix == null) {
                return allFiles;
            }
            else {
                ArrayList selected = new ArrayList();
                for(int i=0; i < allFiles.length; i++) {
                    if(allFiles[i].endsWith(suffix)) {
                        selected.add(allFiles[i]);
                    }
                }
                return (String[])selected.toArray(new String[0]);
            }
        }
        else {
            System.out.println("Error: " + dir + " must be a directory");
            return null;
        }
    }
    

    // ------- ChangeListener interface (for Slider) -------

    /**
     * ChangeListener method for slider changes. This method is called
     * when the slider value is changed by the user.
     */
    public void stateChanged(ChangeEvent evt)
    {
        Object source = evt.getSource();
        if(source == slider) {
            player.seek(slider.getValue());
        }
        else if(source == volumeSlider) {
            player.setVolume(volumeSlider.getValue());
        }
    }
    
    // ------- ActionListener interface (for ComboBox) -------

    /**
     * ActionListener method for changes of format combo box.
     * When this methosd is called, the user has made a new selection 
     * in the combo box.
     */
    public void actionPerformed(ActionEvent evt) 
    {
        JComboBox cb = (JComboBox)evt.getSource();
        format = (String)cb.getSelectedItem();
        if(format.equals("all formats")) {
            format = null;
        }
        reload();
    }

    // ---- Swing stuff to build the frame and all its components and menus ----
    
    /**
     * Create the complete application GUI.
     */
    private void makeFrame()
    {
        // the following makes sure that our application exits when
        // the user closes its window
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        
        JPanel contentPane = (JPanel)getContentPane();
        contentPane.setBorder(new EmptyBorder(6, 10, 10, 10));

        makeMenuBar();
        
        // Specify the layout manager with nice spacing
        contentPane.setLayout(new BorderLayout(8, 8));

        // Create the left side with combobox and scroll list
        JPanel leftPane = new JPanel();
        {
            leftPane.setLayout(new BorderLayout(8, 8));

            String[] formats = { "all formats", ".wav", ".au", ".aif" };
    
            // Create the combo box.
            JComboBox formatList = new JComboBox(formats);
            formatList.addActionListener(this);
            leftPane.add(formatList, BorderLayout.NORTH);
    
            // Create the scrolled list for file names
            fileList = new JList();
            fileList.setForeground(new Color(140,171,226));
            fileList.setBackground(new Color(0,0,0));
            fileList.setSelectionBackground(new Color(87,49,134));
            fileList.setSelectionForeground(new Color(140,171,226));            
            MouseListener mouseListener = new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    playButton.setEnabled(true);
                    if (e.getClickCount() == 2) {
                        play();
                    }
                }
            };
            fileList.addMouseListener(mouseListener);
            reload(); //make sure the files are loaded
            JScrollPane scrollPane = new JScrollPane(fileList);
            scrollPane.setColumnHeaderView(new JLabel("Audio files"));
            leftPane.add(scrollPane, BorderLayout.CENTER);
      
            
            // Create the reload button
            JButton reloadButton = new JButton("Reload");
            reloadButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    reload(); }
            });
            leftPane.add(reloadButton, BorderLayout.SOUTH);
        }
        contentPane.add(leftPane, BorderLayout.WEST);

        // Create the center with image, text label, and slider
       JPanel centerPane = new JPanel();
        {
            centerPane.setLayout(new BorderLayout(8, 8));
    
            JLabel image = new JLabel(new ImageIcon("title.jpg"));
            centerPane.add(image, BorderLayout.NORTH);
            centerPane.setBackground(Color.BLACK);

            infoLabel = new JLabel("  ");
            infoLabel.setHorizontalAlignment(SwingConstants.CENTER);
            infoLabel.setForeground(new Color(140,171,226));
            centerPane.add(infoLabel, BorderLayout.CENTER);

            slider = new JSlider(0, 100, 0);
            //slider.setOpaque(true);
            TitledBorder border = new TitledBorder("Seek");
            border.setTitleColor(Color.white);
            slider.setBorder(new CompoundBorder(new EmptyBorder(6, 10, 10, 10), border));
            slider.addChangeListener(this);
            slider.setBackground(Color.BLUE);
            slider.setOpaque(true);
            slider.setPaintTicks(true);
            slider.setPaintLabels(true);
            Dictionary table = slider.createStandardLabels(25);
            slider.setLabelTable(table);
            centerPane.add(slider, BorderLayout.SOUTH);
            
            
        }
        contentPane.add(centerPane, BorderLayout.CENTER);
        
        // Create the volume control slider
        volumeSlider = new JSlider(JSlider.VERTICAL, 0, 100, 100);
        TitledBorder border = new TitledBorder("Vol");
        volumeSlider.setBorder(new CompoundBorder(new EmptyBorder(10, 10, 10, 10), border));
        volumeSlider.addChangeListener(this);
        volumeSlider.setPaintTicks(true);
        contentPane.add(volumeSlider, BorderLayout.EAST);
        
        // Create the toolbar with the buttons
        JPanel toolbar = new JPanel();
        {
            toolbar.setLayout(new GridLayout(1, 0));
  
            playButton.addActionListener(new ActionListener() {
                                   public void actionPerformed(ActionEvent e) { play(); }
                               });
            toolbar.add(playButton);
            
            stopButton.addActionListener(new ActionListener() {
                                   public void actionPerformed(ActionEvent e) { stop(); }
                               });
            toolbar.add(stopButton);
    
            pauseButton.addActionListener(new ActionListener() {
                                   public void actionPerformed(ActionEvent e) { pause(); }
                               });
            toolbar.add(pauseButton);
            
            resumeButton.addActionListener(new ActionListener() {
                                   public void actionPerformed(ActionEvent e) { resume(); }
                               });
            toolbar.add(resumeButton);
        }
        
        contentPane.add(toolbar, BorderLayout.NORTH);

        // building is done - arrange the components      
        pack();
        
        // place this frame at the center of the screen and show
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(d.width/2 - getWidth()/2, d.height/2 - getHeight()/2);
        setVisible(true);
    }
    
    /**
     * Create the main frame's menu bar.
     */
    private void makeMenuBar()
    {
        final int SHORTCUT_MASK =
            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

        JMenuBar menubar = new JMenuBar();
        setJMenuBar(menubar);
        
        JMenu menu;
        JMenuItem item;
        
        // create the File menu
        menu = new JMenu("File");
        menubar.add(menu);
        
        item = new JMenuItem("Quit");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, SHORTCUT_MASK));
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { quit(); }
                           });
        menu.add(item);
        
                
        item = new JMenuItem("Open");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, SHORTCUT_MASK));
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { open(); }
                           });
        menu.add(item);

        // put a spacer into the menubar, so the next menu appears to the right
        menubar.add(Box.createHorizontalGlue());

        // create the Help menu
        menu = new JMenu("Help");
        menubar.add(menu);
        
        item = new JMenuItem("About SoundPlayer...");
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { showAbout(); }
                           });
        menu.add(item);
    }

    private void disableButtons() {
        playButton.setEnabled(false);
        stopButton.setEnabled(false);
        pauseButton.setEnabled(false);
        resumeButton.setEnabled(false);
    }
}
